# Copyright 2023 L. Nagy, Miguel A. Valdez-Grijalva, W. Williams, A. Muxworthy,  G. Paterson and L. Tauxe
#
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
# following conditions are met:
#
#   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
#      following disclaimer.
#
#   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
#      following disclaimer in the documentation and/or other materials provided with the distribution.
#
#   3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote
#      products derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#
# Project: synth-forc
# File: response.py
# Authors: L. Nagy, Miguel A. Valdez-Grijalva, W. Williams, A. Muxworthy,  G. Paterson and L. Tauxe
# Date: Feb 9 2023
#

from enum import Enum

import schematics


class ResponseStatusEnum(Enum):
    SUCCESS = "SUCCESS"
    EMPTY_LOOPS = "EMPTY_LOOPS"
    EXCEPTION = "EXCEPTION",
    EMPTY_BINS = "EMPTY_BINS"


class Response(schematics.models.Model):

    status = schematics.types.StringType(choices=[ResponseStatusEnum.EXCEPTION.value,
                                                  ResponseStatusEnum.EMPTY_LOOPS.value,
                                                  ResponseStatusEnum.EXCEPTION.value,
                                                  ResponseStatusEnum.EMPTY_BINS.value],
                                         required=True)
    message = schematics.types.StringType(required=True)
    forc_png = schematics.types.StringType(required=True,
                                           default=None,
                                           deserialize_from="forc-png",
                                           serialized_name="forc-png")
    forc_loop_png = schematics.types.StringType(required=True,
                                                default=None,
                                                deserialize_from="forc-loop-png",
                                                serialized_name="forc-loop-png")
    exception = schematics.types.StringType()